/*
	LUSScreenSaver
	Copyright (C) 2003-2004 ONO, Masayuki <mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "../LUSScreenSaver/LUSScreenSaverMisc.h"
#include "LUSScreenSaverPassInputDlg.h"
#include <qapplication.h>
#include <qpe/config.h>
#include "unistd.h"

static const char* LUS_SALT =
#include "../LUSScreenSaver.salt"
;

using namespace LUSMisc;

LUSPassInputDlg::LUSPassInputDlg( QWidget* parent, const char* name )
	: QWidget( parent, name, WDestructiveClose )
	, m_bAuth( 0 )
{
	if ( ! name )
	{
		setName( "LUSPassInputDlg" );
	}

	setBackgroundColor( QColor( 0, 0, 0) );
	erase();

	EmptyLayout = new QGridLayout( this, 3, 3 );

	BaseWidget = new QFrame( this, "BaseWidget" );
	Spacer[0] = new QFrame( this, "Spacer0" );
	Spacer[1] = new QFrame( this, "Spacer1" );
	Spacer[0]->setBackgroundColor( QColor( 0, 0, 0 ) );
	Spacer[1]->setBackgroundColor( QColor( 0, 0, 0 ) );
	Spacer[0]->erase();
	Spacer[1]->erase();
	
	int desktopX = QPEApplication::desktop()->width();
	int desktopY = QPEApplication::desktop()->height();
	Spacer[0]->resize( QSize( desktopX / 3, desktopY / 3 ) );
	Spacer[1]->resize( QSize( desktopX / 3, desktopY / 3 ) );

	EmptyLayout->addWidget( Spacer[0], 0, 1, Qt::AlignCenter );
	EmptyLayout->addWidget( Spacer[1], 1, 0, Qt::AlignLeft );
	EmptyLayout->addWidget( BaseWidget, 1, 1, Qt::AlignCenter );

	BaseLayout = new QVBoxLayout( BaseWidget );
	BaseLayout->setSpacing( 6 );
	BaseLayout->setMargin( 11 );
	
	ButtonsLayout = new QHBoxLayout( BaseWidget );
	ButtonsLayout->setSpacing( 6 );
	ButtonsLayout->setMargin( 11 );

	PassLabel = new QLabel( "Password", BaseWidget, "PassLabel" );
	PassEdit = new QLineEdit( BaseWidget, "PassEdit" );
	PassEdit->setEchoMode( QLineEdit::Password );

	OkButton = new QPushButton( BaseWidget, "OkButton" );
	OkButton->setText( "OK" );
	CancelButton = new QPushButton( BaseWidget, "CancelButton" );
	CancelButton->setText( "CANCEL" );

	ButtonsLayout->addWidget( OkButton );
	ButtonsLayout->addWidget( CancelButton );

	BaseLayout->addWidget( PassLabel );
	BaseLayout->addWidget( PassEdit );
	BaseLayout->addLayout( ButtonsLayout );

	m_pSysChannel = new QCopChannel( "QPE/System", this );
	connect( m_pSysChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveSysMessage( const QCString&, const QByteArray& ) ) );

	connect( OkButton, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( CancelButton, SIGNAL( clicked() ), this, SLOT( close() ) );
}

LUSPassInputDlg::~LUSPassInputDlg()
{
	// MY GOD ...
	DBG_LOG( "LUSPassInputDlg::~LUSPassInputDlg()" );
	delete m_pSysChannel;
}

void LUSPassInputDlg::accept()
{
	Config cfg( "LUSScreenSaver" );
	cfg.setGroup( "Misc" );
	QString enc_pass = LUS_SALT;
	enc_pass += "$";
	QString conf_pass = cfg.readEntry( "Password", QString::null );
	enc_pass += conf_pass;

	QString inp_pass = crypt( PassEdit->text().latin1(), LUS_SALT );

	if ( ( conf_pass.isEmpty() ? PassEdit->text().isEmpty() : FALSE ) || enc_pass == inp_pass )
	{
		m_bAuth = TRUE;
	}
	else
	{
		m_bAuth = FALSE;
/*
		QMessageBox mb( AppName(),
						tr( "AUTH DENIED" ),
						QMessageBox::NoIcon,
						QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape,
						QMessageBox::NoButton,
						QMessageBox::NoButton,
						TaskBarWidget() );
		mb.setIconPixmap( Resource::loadPixmap( "LUSScreenSaver_about" ) );
		mb.adjustSize();

		QPoint oDSize( QPEApplication::desktop()->width(),
		QPEApplication::desktop()->height() );
		QPoint oASize( mb.width(), mb.height() );
		mb.move( ( oDSize -  oASize ) / 2 );
		mb.exec();
*/
	}
	close();
}

void LUSPassInputDlg::closeEvent( QCloseEvent* e )
{
	DBG_LOG( "LUSPassInputDlg::closeEvent()" );
	if ( ! m_bAuth )
	{
		QCopEnvelope en( "QPE/LUSScreenSaver", "SetScreenSaverActivate()" );
		e->ignore();
	}
	else
	{
		emit done();
		e->accept();
	}
}

void LUSPassInputDlg::receiveSysMessage( const QCString& msg, const QByteArray& data )
{
	if ( msg == "appForeground(QString)" || msg == "rotateChangedDone()" )
	{
		DBG_LOG( "LUSPassInputDlg::receiveSysMessage() > raise()" );
		showNormal();
		raise();
		showFullScreen();
	}
}

void LUSPassInputDlg::CheckStart()
{
	showFullScreen();
	setWFlags( WStyle_StaysOnTop );
	setFocus();
	QPEApplication::grabKeyboard();
}

void LUSPassInputDlg::keyPressEvent( QKeyEvent* e )
{
	if ( e->key() == Key_Return )
	{
		accept();
	}
	else if ( e->key() == Key_Escape )
	{
		close();
	}
}

